﻿using EdmLib;
using SldWorks;
using SwConst;
using System;
using System.Runtime.InteropServices;

namespace EPDMCrash
{
	[Guid("AB4E70C2-7083-4476-9E4B-E5DACD5F7983"), ComVisible(true)]
	public class EPDMCrash : IEdmAddIn5
	{
		private static IEdmVault5 vault;
		private static SldWorks.SldWorks swApp;
		private int modelFileID;
		private int modelFolderID;

		void EdmLib.IEdmAddIn5.GetAddInInfo(ref EdmAddInInfo poInfo, IEdmVault5 poVault, IEdmCmdMgr5 poCmdMgr)
		{
			poInfo.mbsAddInName = "EPDM Crash Test";
			poInfo.mbsCompany = "AZZ WSI LLC";
			poInfo.mbsDescription = "Addin demonstrating Access Violation Crash";
            poInfo.mlAddInVersion = 1;
			poInfo.mlRequiredVersionMajor = 22;
			poInfo.mlRequiredVersionMinor = 0;
			vault = poVault;
			poCmdMgr.AddHook(EdmCmdType.EdmCmd_PostState, null);
		}
		void EdmLib.IEdmAddIn5.OnCmd(ref EdmCmd poCmd, ref Array ppoData)
		{
            EdmCmdData[] cmdData = (EdmCmdData[])ppoData;
            if (poCmd.meCmdType == EdmCmdType.EdmCmd_PostState)
            {
                foreach (EdmCmdData d in cmdData)
                {
                    string destStateName = d.mbsStrData2;
                    if (destStateName.Equals("Automatic Processing"))
                    {
                        this.modelFileID = d.mlObjectID1;
                        this.modelFolderID = d.mlObjectID2;
                        ModelDoc2 dwgDoc = getSolidWorks().GetOpenDocument(vault.RootFolderPath + "\\Admin\\EPDMCrash\\foodir\\foo.slddrw");
                        getSolidWorks().FileCloseNotify += processModel;
                        swCloseReopenOption_e options = swCloseReopenOption_e.swCloseReopenOption_DiscardChanges;
                        int result = getSolidWorks().CloseAndReopen2(dwgDoc, (int)options, out object newDoc);
                        getSolidWorks().FileCloseNotify -= processModel;
                    }
                }
            }
        }

        // Event handler for the FileCloseNotify event
        private int processModel(string filePath, int reason)
        {
            if (reason != (int)swFileCloseNotifyReason_e.swFileCloseNotifyReason_CloseForReload) { return 0; }
            IEdmFile6 modelFile = (IEdmFile6)vault.GetObject(EdmObjectType.EdmObject_File, modelFileID);
            string origPath = modelFile.GetLocalPath(modelFolderID);
            ModelDoc2 mDoc = getSolidWorks().GetOpenDocument(origPath);
            ModelDocExtension mExt = mDoc.Extension;
            mDoc.ForceReleaseLocks();
            //Rename and move the file
            string currentModelName = modelFile.Name;
            IEdmFolder5 fooFolder = vault.GetFolderFromPath(vault.RootFolderPath + "\\Admin\\EPDMCrash\\foodir");
            IEdmFolder5 barFolder = vault.GetFolderFromPath(vault.RootFolderPath + "\\Admin\\EPDMCrash\\bardir");
            string newModelName = currentModelName.ToLower().Equals("foo.sldprt") ? "bar.sldprt" : "foo.sldprt";
            modelFile.RenameEx(0, newModelName, 0);
            int currentFolderID = currentModelName.ToLower().Equals("foo.sldprt") ? fooFolder.ID : barFolder.ID;
            int newFolderID = currentModelName.ToLower().Equals("foo.sldprt") ? barFolder.ID : fooFolder.ID;
            modelFile.Move(0, currentFolderID, newFolderID, 0);
            //Reload the file
            string currentPrefix = origPath.Contains("foodir") ? "foo" : "bar";
            string newPrefix = currentPrefix.Equals("foo") ? "bar" : "foo";
            string newPath = origPath.Replace(currentPrefix, newPrefix);
            mExt.ReloadOrReplace(true, newPath, true, false);
            getSolidWorks().SetPromptFilename2(filePath, "");
            return 0;
        }
        //Retrieve the running SOLIDWORKS instance
        private SldWorks.SldWorks getSolidWorks()
        {
            if (swApp == null)
            {
                swApp = Marshal.GetActiveObject("SldWorks.Application") as SldWorks.SldWorks;
            }
            return swApp;
        }
    }
}
